@@ -71,8 +71,9 @@ from utils import trace, get_version
 #
 class FunkLoadXmlParser:
     """Parse a funkload xml results."""
-    def __init__(self):
+    def __init__(self, apdex_t):
         """Init setup expat handlers."""
+        self.apdex_t = apdex_t
         parser = xml.parsers.expat.ParserCreate()
         parser.CharacterDataHandler = self.handleCharacterData
         parser.StartElementHandler = self.handleStartElement
@@ -105,7 +106,7 @@ class FunkLoadXmlParser:
                     self.current_element[1]['name'] != 'funkload'):
                     print """Note that you can generate a report only for a
                     bench result done with fl-run-bench (and not on a test
-                    result done with fl-run-test)."""
+                    resu1lt done with fl-run-test)."""
                 else:
                     print """You may need to remove non ascii characters which
                     come from error pages caught during the bench test. iconv
@@ -149,12 +150,13 @@ class FunkLoadXmlParser:
             stats = self.stats.setdefault(cycle, {'response_step':{}})
             stat = stats.setdefault(
                 'response', AllResponseStat(cycle, self.cycle_duration,
-                                            attrs['cvus']))
+                                            attrs['cvus'], self.apdex_t))
             stat.add(attrs['time'], attrs['result'], attrs['duration'])
             stats['response'] = stat
 
             stat = stats.setdefault(
-                'page', PageStat(cycle, self.cycle_duration, attrs['cvus']))
+                'page', PageStat(cycle, self.cycle_duration, attrs['cvus'], 
+                                 self.apdex_t))
             stat.add(attrs['thread'], attrs['step'], attrs['time'],
                      attrs['result'], attrs['duration'], attrs['type'])
             stats['page'] = stat
@@ -162,7 +164,7 @@ class FunkLoadXmlParser:
             step = '%s.%s' % (attrs['step'], attrs['number'])
             stat = stats['response_step'].setdefault(
                 step, ResponseStat(attrs['step'], attrs['number'],
-                                   attrs['cvus']))
+                                   attrs['cvus'], self.apdex_t))
             stat.add(attrs['type'], attrs['result'], attrs['url'],
                      attrs['duration'], attrs.get('description'))
             stats['response_step'][step] = stat
@@ -229,8 +231,11 @@ def main():
                       dest="report_dir",
                       help="Directory name to store the report.",
                       default=None)
-
-
+    parser.add_option("-T", "--apdex-T", type="float",
+                      dest="apdex_t",
+                      help="Apdex T constant in second, default is set to 1.5s. "
+                      "Visit http://www.apdex.org/ for more information.",
+                      default=1.5)
 
     options, args = parser.parse_args()
     if options.diffreport:
@@ -253,7 +258,7 @@ def main():
             trace("Results merged in tmp file: %s\n" % os.path.abspath(tmp_file))
             args = [tmp_file]
         options.xml_file = args[0]
-        xml_parser = FunkLoadXmlParser()
+        xml_parser = FunkLoadXmlParser(options.apdex_t)
         xml_parser.parse(options.xml_file)
         if options.html:
             trace("Creating html report: ...")
