@@ -1,7 +1,7 @@
 # -*- coding: iso-8859-15 -*-
 """Simple FunkLoad test
 
-$Id: test_Simple.py 53544 2009-03-09 16:28:58Z tlazar $
+$Id: test_Simple.py 54228 2010-04-14 22:11:52Z bdelbosc $
 """
 import unittest
 from random import random
@@ -12,7 +12,6 @@ class Simple(FunkLoadTestCase):
 
     def setUp(self):
         """Setting up test."""
-        self.logd("setUp")
         self.server_url = self.conf_get('main', 'url')
 
     def test_simple(self):
@@ -20,19 +19,10 @@ class Simple(FunkLoadTestCase):
         server_url = self.server_url
         # begin of test ---------------------------------------------
         nb_time = self.conf_getInt('test_simple', 'nb_time')
-        pages = self.conf_getList('test_simple', 'pages')
-
         for i in range(nb_time):
-            self.logd('Try %i' % i)
-            for page in pages:
-                self.get(server_url + page, description='Get %s' % page)
-
+            self.get(server_url, description='Get url')
         # end of test -----------------------------------------------
 
-    def tearDown(self):
-        """Setting up test."""
-        self.logd("tearDown.\n")
-
 
 if __name__ in ('main', '__main__'):
     unittest.main()
